#pragma once

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <io.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>

#pragma warning(disable:4786)  // VisualC++6.0̌x΍
#include <ios> 
#include <map>
#include <string>
#include <iostream>
#include <vector>

using namespace std;

#define	NOW_MAX		30
#define SVC_MAX		300
#define TABLE_MAX	2
#define DATAFILEVER	1

/* }N` */
#define GetNetId(r) (r/4096)

static void Convert_jpn_arib_char(char *str,size_t len,char *ret_buf,size_t ret_len);
static void Convert_mjd_to_date(unsigned short mjd,long *p_year,long *p_month,long *p_day);
static void BinaryDump(BYTE* buf,int size);
static void MsgOut(LPCTSTR szFormat, ...);

static void MsgOut(LPCTSTR szFormat, ...)
{
	return;
	va_list Args;
	TCHAR szTempStr[4096];

	va_start(Args , szFormat);
	wvsprintf(szTempStr, szFormat, Args);
	va_end(Args);

	OutputDebugString(szTempStr);
}
static void BinaryDump(BYTE* buf,int size)
{
	char szmsg[1024];
	szmsg[0] = 0x00;
	int x = 0;
	for(int i = 0;i < size;i++) {
		if(x == 0) {
			if(strlen(szmsg) > 0) {
				strcat(szmsg,"\r\n");
				OutputDebugString(szmsg);
				szmsg[0] = 0x00;
			}
			sprintf(szmsg,"%04x:",i);
		}
		else
			strcat(szmsg,",");
		sprintf(&szmsg[strlen(szmsg)],"%02x",(BYTE)buf[i]);
		x++;
		if(x >= 16)
			x = 0;
	}
	if(strlen(szmsg) > 0) {
		strcat(szmsg,"\r\n");
		OutputDebugString(szmsg);
		szmsg[0] = 0x00;
	}
}

static void Convert_mjd_to_date(unsigned short mjd,long *p_year,long *p_month,long *p_day)
{
	double ajd = (double)mjd + 2400000.5;
	double t=ajd+0.5;
	long   z=(long)t;
	double f=t-z;
	long a;
	if(z<2299161)
	{
		a=z;
	} else {
		long x=(long)((z-1867216.25)/36524.25);
		a=z+1+x-(x/4); 
	}
	double b=a+1524;
	double c=(long)((b-122.1)/365.25);
	double d=(long)(365.25*c);
	double e=(long)((b-d)/30.6001);
	*p_day  =(long)(b-d-(long)(30.6001*e)+f);
	*p_month=(long)(e<14?e-1:e-13);
	*p_year =(long)(*p_month>2?c-4716:c-4715);
}

// ref ARIB STD-B24v1

#define CODE_JPN_KANJI            0
#define CODE_JPN_ALPHANUMERIC     1
#define CODE_JPN_HIRAGANA         2
#define CODE_JPN_KATAKANA         3
#define CODE_JPN_MOSAIC_A         4
#define CODE_JPN_MOSAIC_B         5
#define CODE_JPN_MOSAIC_C         6
#define CODE_JPN_MOSAIC_D         7
#define CODE_JPN_PRO_ALPHANUMERIC 8
#define CODE_JPN_PRO_HIRAGANA     9
#define CODE_JPN_PRO_KATAKANA     10
#define CODE_JPN_X0201_KATAKANA   11
#define CODE_JPN_JIS_KANJI_1      12
#define CODE_JPN_JIS_KANJI_2      13
#define CODE_JPN_ADDITIONAL       14

#define CODE_JPN_MODE_NONE 0
#define CODE_JPN_MODE_GL   1
#define CODE_JPN_MODE_GR   2

static char *jpn_argb_alp_num_table = 
/*      0 1 2 3 4 5 6 7 8 9 A B C D E F */
/*0x*/"@@@@@@@@@@@@@@@@" 
/*1x*/"@@@@@@@@@@@@@@@@" 
/*2x*/"@Ihfij{C|D^" 
/*3x*/"OPQRSTUVWXFGH" 
/*4x*/"`abcdefghijklmn" 
/*5x*/"opqrstuvwxymnOQ" 
/*5x*/"@" 
/*6x*/"obpP@" ;

static char *jpn_argb_alp_num_table_h = 
/*     0123456789ABCDEF */
/*0x*/"                " 
/*1x*/"                " 
/*2x*/" !\"#$%&'()*+,-./" 
/*3x*/"0123456789:;<=>?" 
/*4x*/"@ABCDEFGHIJKLMNO" 
/*5x*/"PQRSTUVWXYZ[\\]^_" 
/*5x*/" abcdefghijklmno" 
/*6x*/"pqrstuvwxyz{|}~";

static char *jpn_argb_hiragana_table = 
/*      0 1 2 3 4 5 6 7 8 9 A B C D E F */
/*0x*/"@@@@@@@@@@@@@@@@" 
/*1x*/"@@@@@@@@@@@@@@@@" 
/*2x*/"@" 
/*3x*/"" 
/*4x*/"ÂĂłƂǂȂɂʂ˂̂" 
/*5x*/"΂ςЂт҂ӂԂՂւׂ؂قڂۂ܂" 
/*6x*/"ނ߂"
/*7x*/"@@@TU[BuvAE@" ;

static char *jpn_argb_katakana_table = 
/*      0 1 2 3 4 5 6 7 8 9 A B C D E F */
/*0x*/"@@@@@@@@@@@@@@@@" 
/*1x*/"@@@@@@@@@@@@@@@@" 
/*2x*/"@@ABCDEFGHIJKLMN" 
/*3x*/"OPQRSTUVWXYZ[\]^" 
/*4x*/"_`abcdefghijklmn" 
/*5x*/"opqrstuvwxyz{|}~" 
/*6x*/""
/*7x*/"RS[BuvAE@" ;

static char *jpn_argb_x0201_katakana_table = 
/*      0 1 2 3 4 5 6 7 8 9 A B C D E F */
/*0x*/"@@@@@@@@@@@@@@@@" 
/*1x*/"@@@@@@@@@@@@@@@@" 
/*2x*/"@BuvAE@BDFHb"
/*3x*/"[ACEGIJLNPRTVXZ\" 
/*4x*/"^`cegijklmnqtwz}" 
/*5x*/"~JK" 
/*6x*/"@@@@@@@@@@@@@@@@"
/*7x*/"@@@@@@@@@@@@@@@@" ;
static int jpn_argb_original_table_sel[100] = {0x7a50,0x7a74,0x7d6e,0x7d6f,0x7e21,0x7e7d,0x753f,0x7541,0x7544,0x7544,0x7647,0x7647,0x763d,0x763d,0,0};
static char* jpn_argb_original_table_str[100][100] = {
	{"[HV]","[SD]","[o]","[v]","[MV]","[]","[]","[o]","[f]","[r]","[]","[]","[]","[SS]","[a]","[m]","","","[V]","[]","[f]","[]","[]","_","[O]","[]","[]","[V]","[]","[I]","[]","[]","[]","[]","[PPV]","()","<ق>",0},
	{"!!","!?"},
	{"T","U","V","W","X","Y","Z","[","\","]","]T","]U","(17)","(18)","(19)","(20)","(1)","(2)","(3)","(4)","(5)","(6)","(7)","(8)","(9)","(10)","(11)","(12)","(21)","(22)","(23)","(24)","(A)","(B)","(C)","(D)","(E)","(F)","(G)","(H)","(I)","(J)","(K)","(L)","(M)","(N)","(O)","(P)","(Q)","(R)","(S)","(T)","(U)","(V)","(W)","(X)","(Y)","(Z)","(25)","(26)","(27)","(28)","(29)","(30)","@","A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","<1>","<2>","<3>","<4>","<5>","<6>","<7>","<8>","<9>","<11>","<11>","<12>","(31)"},
	{"","",""},
	{"g"},
	{""},
	{"A"}};

static void Convert_jpn_arib_char(char *str,size_t len,char *ret_buf,size_t ret_len)
{
	unsigned char *p=(unsigned char*)str;
	size_t pos,ret_pos,write_size;	
	unsigned char c1,c2;
	char szAdd[10];

	unsigned char typG0 = CODE_JPN_KANJI;
	unsigned char typG1 = CODE_JPN_ALPHANUMERIC;
	unsigned char typG2 = CODE_JPN_HIRAGANA;
	unsigned char typG3 = CODE_JPN_KATAKANA;

	unsigned char *modGL  = &typG0;
	unsigned char *modGR  = &typG2;
	unsigned char *modGLs = NULL; // SingleShift
	unsigned char modeLR,mode;

	BOOL b1byte = FALSE;
	// convert 2byte code
	pos=ret_pos=0;
	for(pos=0;pos<len&&p[pos];)
	{
		char code_len=1;
		c1=c2=0;
		modeLR=CODE_JPN_MODE_NONE;
		mode=0;
		write_size=0;
		modGLs=NULL;
		szAdd[0] = 0x00;
		unsigned char *sel_type=NULL;
		unsigned char new_type=0;
		if( p[pos] <= 0x20 )
		{
			// C0 Area
			switch( p[pos] )
			{
			case 0x0F: modGL = &typG0;	break;
			case 0x0E: modGL = &typG1;	break;
			case 0x1B:
				{
					// ESC
					code_len=2;
					switch(p[pos+1])
					{
					case 0x6E: modGL = &typG2; break;
					case 0x6F: modGL = &typG3; break;
					case 0x7E: modGR = &typG1; break;
					case 0x7D: modGR = &typG2; break;
					case 0x7C: modGR = &typG3; break;
					case 0x28:
						{
							sel_type = &typG0;
							if(p[pos+2]!=0x20)
							{
								new_type = p[pos+3]; code_len=4;
							} else {
								new_type = p[pos+2]; code_len=3;
							}									
						}
						break;
					case 0x29:
						{
							sel_type = &typG1;
							if(p[pos+2]==0x20)
							{
								new_type = p[pos+3]; code_len=4;
							} else {
								new_type = p[pos+2]; code_len=3;
							}
						}
						break;
					case 0x2A:
						{
							sel_type = &typG2;
							if(p[pos+2]==0x20)
							{
								new_type = p[pos+3]; code_len=4;
							} else {
								new_type = p[pos+2]; code_len=3;
							}
						}
						break;
					case 0x2B:
						{
							sel_type = &typG3;
							if(p[pos+2]==0x20)
							{
								new_type = p[pos+3]; code_len=4;
							} else {
								new_type = p[pos+2]; code_len=3;
							}							
						}
						break;
					case 0x24:
						{
							switch(p[pos+2])
							{
							case 0x28:
								{
									sel_type = &typG0;
									if(p[pos+3]==0x20)
									{
										new_type = p[pos+4]; code_len=5;
									} else {
										sel_type = NULL;
									}
								}
								break;
							case 0x29:
								{
									sel_type = &typG1;
									if(p[pos+3]==0x20)
									{
										new_type = p[pos+4]; code_len=5;
									} else {
										new_type = p[pos+3]; code_len=4;
									}
								}
								break;
							case 0x2A:
								{
									sel_type = &typG2;
									if(p[pos+3]==0x20)
									{
										new_type = p[pos+4]; code_len=5;
									} else {
										new_type = p[pos+3]; code_len=4;
									}
								}
								break;
							case 0x2B:
								{
									sel_type = &typG3;
									if(p[pos+3]==0x20)
									{
										new_type = p[pos+4]; code_len=5;
									} else {
										new_type = p[pos+3]; code_len=4;
									}
								}
								break;
							default:
								{
									sel_type = &typG0;
									new_type = p[pos+2]; code_len=3;
								}
							}
						}
						break;
					}					
					break;					
				}
				break;
			case 0x19: modGLs = &typG2; break;
			case 0x1D: modGLs = &typG3; break;
			}
			if(sel_type)
			{
				switch(new_type)
				{
				case 0x42: *sel_type=CODE_JPN_KANJI;            break;
				case 0x4A: *sel_type=CODE_JPN_ALPHANUMERIC;     break;
				case 0x30: *sel_type=CODE_JPN_HIRAGANA;         break;
				case 0x31: *sel_type=CODE_JPN_KATAKANA;         break;
				case 0x32: *sel_type=CODE_JPN_MOSAIC_A;         break;
				case 0x33: *sel_type=CODE_JPN_MOSAIC_B;         break;
				case 0x34: *sel_type=CODE_JPN_MOSAIC_C;         break;
				case 0x35: *sel_type=CODE_JPN_MOSAIC_D;         break;
				case 0x36: *sel_type=CODE_JPN_PRO_ALPHANUMERIC; break;			
				case 0x37: *sel_type=CODE_JPN_PRO_HIRAGANA;     break;
				case 0x38: *sel_type=CODE_JPN_PRO_KATAKANA;     break;
				case 0x49: *sel_type=CODE_JPN_X0201_KATAKANA;   break;
				case 0x39: *sel_type=CODE_JPN_JIS_KANJI_1;      break;				
				case 0x3A: *sel_type=CODE_JPN_JIS_KANJI_2;      break;
				case 0x3B: *sel_type=CODE_JPN_ADDITIONAL;       break;
				default: break;
				}
			}
			write_size=0;
			if(p[pos] == 0x0d) {
				c1 = 0x0d;
				write_size = 1;
				if(p[pos+1] == 0x0a)
					code_len = 2;
			}
		} else if( p[pos] >= 0x21 && p[pos] <= 0x7E ) {
			// GL Area
			modeLR = CODE_JPN_MODE_GL;
			mode=*modGL;
		} else if( p[pos] >= 0x7F && p[pos] <= 0xA0 ) {
			// C1 Area
			write_size=0;
			if(p[pos] == 0x89)
				b1byte = TRUE;
			else if(p[pos]== 0x8a)
				b1byte = FALSE;
		} else {
			// GR Area
			modeLR = CODE_JPN_MODE_GR;
			mode=*modGR;
		}
		if(modGLs||modeLR!=CODE_JPN_MODE_NONE)
		{
			unsigned char code_shift=0;
			switch(modeLR)
			{
			case CODE_JPN_MODE_GL:
				{
					code_shift=0x00;
				}
				break;
			case CODE_JPN_MODE_GR:
				{
					code_shift=0x80;
				}
				break;
			}
			if(modGLs)
			{
				code_shift=0x00;
				mode=*modGLs;
				pos++;
			}
			switch(mode)
			{
			case CODE_JPN_KANJI:
			case CODE_JPN_JIS_KANJI_1:
			case CODE_JPN_JIS_KANJI_2:

				{
					char t;
					t=p[pos+1];
					if(p[pos]&0x01)
					{
						c2=t+0x1F;			
					} else {
						c2=t+0x7D;
					}
					t=p[pos];
					if(t<0x5f)
					{
						c1=(t+1)/2+0x70;
					} else {
						c1=(t+1)/2+0xb0;
					}
					if(c2>0x7e) {
						c2++;
					}
					code_len=2;
					write_size=2;
				}
				break;
			case CODE_JPN_ALPHANUMERIC:
				{
					if(!b1byte) {
						c1=jpn_argb_alp_num_table[(p[pos]-code_shift)*2];
						c2=jpn_argb_alp_num_table[(p[pos]-code_shift)*2+1];
						write_size=2;
					}
					else {
						c1=jpn_argb_alp_num_table_h[(p[pos]-code_shift)];
						write_size=1;
					}
				}
				break;
			case CODE_JPN_HIRAGANA:
				{
					c1=jpn_argb_hiragana_table[(p[pos]-code_shift)*2];
					c2=jpn_argb_hiragana_table[(p[pos]-code_shift)*2+1];
					write_size=2;
				}
				break;
			case CODE_JPN_KATAKANA:
				{
					c1=jpn_argb_katakana_table[(p[pos]-code_shift)*2];
					c2=jpn_argb_katakana_table[(p[pos]-code_shift)*2+1];
					write_size=2;
				}
				break;
			case CODE_JPN_ADDITIONAL:
				{
					c1=p[pos+0];
					c2=p[pos+1];
					code_len=2;
					write_size=0;
					szAdd[0] = 0x00;
					WORD orgcode = (((WORD)(c1&0x7f))<<8) + (WORD)(c2&0x7f);
					//wsprintf(szAdd,"%04x)",orgcode);
					//write_size = strlen(szAdd);
					for(int sel = 0;jpn_argb_original_table_sel[sel] != 0;sel+=2) {
						if(jpn_argb_original_table_sel[sel] <= orgcode && jpn_argb_original_table_sel[sel+1] >= orgcode) {
							strcpy(szAdd,jpn_argb_original_table_str[sel/2][orgcode-jpn_argb_original_table_sel[sel]]);
							write_size = strlen(szAdd);
							break;
						}
					}
					//if(write_size == 0)
					//	MsgOut("AribCode(%02x,%02x)\n",c1,c2);
				}
				break;
			}
		}
		if(ret_buf&&ret_pos+write_size+1<ret_len)
		{
			if(szAdd[0]) {
				for(int cnt = 0;cnt < strlen(szAdd);cnt++) {
					ret_buf[ret_pos]=szAdd[cnt];
					ret_pos+=1;
				}
			}
			else {
				switch(write_size)
				{
				case 1:
					{
						ret_buf[ret_pos]=c1;
						ret_pos+=1;
					}
					break;
				case 2:
					{
						ret_buf[ret_pos]=c1;
						ret_buf[ret_pos+1]=c2;
						ret_pos+=2;
					}
					break;
				}
			}
		}
		pos+=code_len;
	}
	if(ret_buf)
	{
		if(ret_pos+1<ret_len)
		{
			ret_buf[ret_pos]='\0';
		} else {		
			ret_buf[ret_len-1]='\0';
		}
	}
}

typedef struct {

	int           pid;
	int           last_continuity_counter;

	__int64       total;
	__int64       drop;
	__int64       scrambling;

	unsigned char last_packet[188];
	int           duplicate_count;

} TS_STATUS;

typedef struct {
	int           sync;
	int           transport_error_indicator;
	int           payload_unit_start_indicator;
	int           transport_priority;
	int           pid;
	int           transport_scrambling_control;
	int           adaptation_field_control;
	int           continuity_counter;
} TS_HEADER;

typedef struct {
	int			table_id;
	int			section_length;
	int			service_id;
	int			version_number;
	int			current_next_indicator;
	int			section_number;
	int			last_section_number;
	int			transport_stream_id;
	int			original_network_id;
	int			segment_last_section_number;
	int			last_table_id;
} EIT_HEADER;

typedef struct tagSecData {
	int			nextcount;
	BYTE*		pData;
	int			bufflen;
	int			seclen;
	int			secrcv;

	tagSecData() {
		memset(this,0,sizeof(SEC_DATA));
	}
	~tagSecData() {
		DeleteData();
	}
	void SetNextCount(int count) {
		nextcount = count;
		nextcount++;
		if(nextcount > 0x0f)
			nextcount = 0;
	}
	int GetNextCount() {
		return nextcount;
	}
	void AddData(BYTE* sbuf,int sbuflen) {
		if(bufflen < secrcv + sbuflen || !pData) {
			BYTE* pDumy = new BYTE[secrcv + sbuflen + 200];
			if(pData) {
				memcpy(pDumy,pData,bufflen);
				delete[] pData;
			}
			pData = pDumy;
			bufflen = secrcv + sbuflen + 200;
		}
		memcpy(&pData[secrcv],sbuf,sbuflen);
		secrcv += sbuflen;
	}
	BYTE* GetData() {
		return pData;
	}
	int GetDataSize() {
		return secrcv;
	}
	void ResetData() {
		seclen = 0;
		secrcv = 0;
	}
	BOOL IsNext() {
		return secrcv != 0;
	}
	void DeleteData() {
		if(pData != NULL)
			delete[] pData;
		pData = NULL;
		bufflen = 0;
		seclen = 0;
		secrcv = 0;
	}
} SEC_DATA;

typedef struct tagEitSecData {
	WORD		ver_cng_count;
	BYTE*		pData;
	WORD		buffsize;
	WORD		datasize;

	tagEitSecData() {
		Reset();
	}
	tagEitSecData(const tagEitSecData& Value) {
		Reset();
		Copy((tagEitSecData*)&Value);
	}
	~tagEitSecData() {
		DeleteData();
	}
	void Reset() {
		memset(this,0,sizeof(EIT_SEC_DATA));
	}
	void Copy(tagEitSecData* pSecData) {
		ver_cng_count = pSecData->ver_cng_count;
		DeleteData();
		if(pSecData->datasize > 0)
			SetData(pSecData->pData,pSecData->datasize);
	}
	int GetServiceId() {
		if(pData)
			return ((pData[3]<<8) + pData[4]);
		return 0;
	}
	void GetDate(long* year,long* month,long* day) {
		*year = 0;
		*month = 0;
		*day = 0;
		if(pData)
			Convert_mjd_to_date((pData[14+2]<<8)+pData[14+3],year,month,day);
	}
	int GetVersion() {
		if(pData)
			return (pData[5]>>1) & 0x1f;
		return -1;
	}
	void SetData(BYTE* pInData,int size) {
		if(buffsize < size) {
			DeleteData();
			pData = new BYTE[size];
			buffsize = size;
		}
		memcpy(pData,pInData,size);
		datasize = size;
	}
	BYTE* GetData() {
		return pData;
	}
	int GetDataSize() {
		return datasize;
	}
	void DeleteData() {
		if(pData != NULL)
			delete[] pData;
		pData = NULL;
		buffsize = 0;
		datasize = 0;
	}
} EIT_SEC_DATA;

typedef struct tagSitSecData {
	WORD		ver_cng_count;
	BYTE		version;
	WORD		service_id;
	WORD		network_id;
	BYTE		date[5];
	BYTE*		pData;
	WORD		buffsize;
	WORD		datasize;

	tagSitSecData() {
		Reset();
	}
	tagSitSecData(const tagSitSecData& Value) {
		Reset();
		Copy((tagSitSecData*)&Value);
	}
	~tagSitSecData() {
		DeleteData();
	}
	void Reset() {
		memset(this,0,sizeof(SIT_SEC_DATA));
	}
	void Copy(tagSitSecData* pSecData) {
		ver_cng_count = pSecData->ver_cng_count;
		version = pSecData->version;
		service_id = pSecData->service_id;
		network_id = pSecData->network_id;
		memcpy(date,pSecData->date,5);
		DeleteData();
		if(pSecData->datasize > 0)
			SetData(pSecData->pData,pSecData->datasize);
	}
	int GetServiceId() {
		if(pData)
			return service_id;
		return 0;
	}
	void GetDate(long* year,long* month,long* day) {
		*year = 0;
		*month = 0;
		*day = 0;
		if(pData)
			Convert_mjd_to_date((date[0]<<8)+pData[1],year,month,day);
	}
	int GetVersion() {
		if(pData)
			return version;
		return -1;
	}
	void SetData(BYTE* pInData,int size) {
		if(buffsize < size) {
			DeleteData();
			pData = new BYTE[size];
			buffsize = size;
		}
		memcpy(pData,pInData,size);
		datasize = size;
	}
	BYTE* GetData() {
		return pData;
	}
	int GetDataSize() {
		return datasize;
	}
	void DeleteData() {
		if(pData != NULL)
			delete[] pData;
		pData = NULL;
		buffsize = 0;
		datasize = 0;
	}
} SIT_SEC_DATA;

typedef struct tagPatData {
	WORD			Service_id;
	WORD			pmtPid;
	WORD			vPid;
	DWORD			vCount;
	EIT_SEC_DATA	nowEit;
	SIT_SEC_DATA	nowSit;

	tagPatData() {
		Service_id = 0;
		pmtPid = 0;
		vPid = 0;
		vCount = 0;
	}
} PAT_DATA;

typedef struct tagPatTable {
	WORD	patVersion;
	vector<PAT_DATA> patTable;

	tagPatTable() {
		patVersion = 0;
		patTable.clear();
	}
} PAT_TABLE;

typedef struct tagSvcData {
	WORD			Service_id;
	WORD			chNo;
	char			szProviderName[50];
	WORD			nProviderNameLen;
	char			szName[50];
	WORD			nNameLen;
	BYTE			remoconNo;
	char			szTsName[50];
	WORD			nTsNameLen;
	EIT_SEC_DATA	EitNowData[2];
	SIT_SEC_DATA	SitNowData;

	//EIT_SEC_DATA	EitSecData[TABLE_MAX][256];
	//EIT_SEC_DATA	EitSecDataEx[TABLE_MAX][256];
	//EIT_SEC_DATA	EitSecDataLast[TABLE_MAX][256];

	tagSvcData() {
		memset(this,0,sizeof(SVC_DATA));
	}
} SVC_DATA;

class CTsData
{
public:
	CTsData(void);
	~CTsData(void);
	void tsread(const char *path,BOOL pCallBack(long readbyte,long totalbyte,HWND data) = NULL,HWND indata = NULL);
	void RecvTsPacket(BYTE* tsbuf);
	void ChangeChannel(void);
	void SetChannel(int ServiceID,int Channel);
	int GetSvcData(int SerivceId,BYTE NetworkId);
	int SearchSvcData(int ServiceId,BYTE NetworkId);
	void DeleteSvcData(int ServiceId,BYTE NetworkId);
	void DataAllClear(BOOL bAll = TRUE);
	void ConvDate(BYTE* pEpgDate,time_t* sTime,time_t* eTime);

	SVC_DATA* SvcData[SVC_MAX];
	int LastSvcData;
	SEC_DATA* SecData[8192];
	int NowYear;
	int NowMonth;
	int NowDay;
	int NowHour;
	int NowMin;
	int NowSec;
	time_t NowTime_t;
	int	DataFileVer;
	BOOL bDataUpdate;
	vector<PAT_TABLE> PatList;
	int ReadPos;
	int ReadSize;

private:
	void secdump(TS_HEADER* ts,BYTE* buf);
	void nitdump(TS_HEADER* ts);
	void pidcheck(TS_HEADER* ts);
	void sdtdump(TS_HEADER* ts);
	void eitdump(TS_HEADER* ts);
	void sitdump(TS_HEADER* ts);
	void patdump(TS_HEADER* ts);
	void pmtdump(TS_HEADER* ts);
	void extract_eit_header(EIT_HEADER *eit,unsigned char *buf);
	void extract_ts_header(TS_HEADER *dst, unsigned char *packet);
	unsigned long Calc_crc32(unsigned char *start, size_t len);
	int select_unit_size(unsigned char *head, unsigned char *tail);
	unsigned char *resync(unsigned char *head, unsigned char *tail, int unit_size);
	unsigned char *resync_force(unsigned char *head, unsigned char *tail, int unit_size);
	int check_unit_invert(unsigned char *head, unsigned char *tail);

	BOOL bEnd;
};
